package org.eclipse.osgi.util;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import java.util.StringTokenizer;
import org.eclipse.osgi.framework.internal.core.Tokenizer;
import org.eclipse.osgi.util.NLS;
import org.osgi.framework.BundleException;

public class ManifestElement {
  private static final String MANIFEST_INVALID_SPACE = "The manifest line \"{0}\" has an invalid leading space '' '' character.";
  
  private static final String MANIFEST_INVALID_LINE_NOCOLON = "The manifest line \"{0}\" is invalid; it has no colon '':'' character after the header key.";
  
  private static final String MANIFEST_INVALID_HEADER_EXCEPTION = "Invalid manifest header {0}: \"{1}\"";
  
  protected String value;
  
  protected String[] valueComponents;
  
  protected Hashtable attributes;
  
  protected Hashtable directives;
  
  public String getValue() {
    return this.value;
  }
  
  public String[] getValueComponents() {
    return this.valueComponents;
  }
  
  public String getAttribute(String key) {
    return getTableValue(this.attributes, key);
  }
  
  public String[] getAttributes(String key) {
    return getTableValues(this.attributes, key);
  }
  
  public Enumeration getKeys() {
    return getTableKeys(this.attributes);
  }
  
  protected void addAttribute(String key, String value) {
    this.attributes = addTableValue(this.attributes, key, value);
  }
  
  public String getDirective(String key) {
    return getTableValue(this.directives, key);
  }
  
  public String[] getDirectives(String key) {
    return getTableValues(this.directives, key);
  }
  
  public Enumeration getDirectiveKeys() {
    return getTableKeys(this.directives);
  }
  
  protected void addDirective(String key, String value) {
    this.directives = addTableValue(this.directives, key, value);
  }
  
  private String getTableValue(Hashtable table, String key) {
    if (table == null)
      return null; 
    Object result = table.get(key);
    if (result == null)
      return null; 
    if (result instanceof String)
      return (String)result; 
    ArrayList<String> valueList = (ArrayList)result;
    return valueList.get(valueList.size() - 1);
  }
  
  private String[] getTableValues(Hashtable table, String key) {
    if (table == null)
      return null; 
    Object result = table.get(key);
    if (result == null)
      return null; 
    if (result instanceof String)
      return new String[] { (String)result }; 
    ArrayList valueList = (ArrayList)result;
    return (String[])valueList.toArray((Object[])new String[valueList.size()]);
  }
  
  private Enumeration getTableKeys(Hashtable table) {
    if (table == null)
      return null; 
    return table.keys();
  }
  
  private Hashtable addTableValue(Hashtable<Object, Object> table, String key, String value) {
    if (table == null)
      table = new Hashtable<Object, Object>(7); 
    Object curValue = table.get(key);
    if (curValue != null) {
      ArrayList<Object> newList;
      if (curValue instanceof ArrayList) {
        newList = (ArrayList)curValue;
      } else {
        newList = new ArrayList(5);
        newList.add(curValue);
      } 
      newList.add(value);
      table.put(key, newList);
    } else {
      table.put(key, value);
    } 
    return table;
  }
  
  public static org.eclipse.osgi.util.ManifestElement[] parseHeader(String header, String value) throws BundleException {
    char c;
    if (value == null)
      return null; 
    ArrayList<org.eclipse.osgi.util.ManifestElement> headerElements = new ArrayList(10);
    Tokenizer tokenizer = new Tokenizer(value);
    while (true) {
      String next = tokenizer.getString(";,");
      if (next == null)
        throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
      ArrayList<String> headerValues = new ArrayList();
      StringBuffer headerValue = new StringBuffer(next);
      headerValues.add(next);
      boolean directive = false;
      c = tokenizer.getChar();
      while (c == ';') {
        next = tokenizer.getString(";,=:");
        if (next == null)
          throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
        c = tokenizer.getChar();
        while (c == ':') {
          c = tokenizer.getChar();
          if (c != '=') {
            String restOfNext = tokenizer.getToken(";,=:");
            if (restOfNext == null)
              throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
            next = String.valueOf(next) + ":" + c + restOfNext;
            c = tokenizer.getChar();
            continue;
          } 
          directive = true;
        } 
        if (c == ';' || c == ',' || c == '\000') {
          headerValues.add(next);
          headerValue.append(";").append(next);
        } 
      } 
      org.eclipse.osgi.util.ManifestElement manifestElement = new org.eclipse.osgi.util.ManifestElement();
      manifestElement.value = headerValue.toString();
      manifestElement.valueComponents = headerValues.<String>toArray(new String[headerValues.size()]);
      while (c == '=' || c == ':') {
        while (c == ':') {
          c = tokenizer.getChar();
          if (c != '=') {
            String restOfNext = tokenizer.getToken("=:");
            if (restOfNext == null)
              throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
            next = String.valueOf(next) + ":" + c + restOfNext;
            c = tokenizer.getChar();
            continue;
          } 
          directive = true;
        } 
        String val = tokenizer.getString(";,");
        if (val == null)
          throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
        try {
          if (directive) {
            manifestElement.addDirective(next, val);
          } else {
            manifestElement.addAttribute(next, val);
          } 
          directive = false;
        } catch (Exception exception) {
          throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
        } 
        c = tokenizer.getChar();
        if (c == ';') {
          next = tokenizer.getToken("=:");
          if (next == null)
            throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3); 
          c = tokenizer.getChar();
        } 
      } 
      headerElements.add(manifestElement);
      if (c == ',')
        continue; 
      break;
    } 
    if (c == '\000') {
      int size = headerElements.size();
      if (size == 0)
        return null; 
      org.eclipse.osgi.util.ManifestElement[] result = headerElements.<org.eclipse.osgi.util.ManifestElement>toArray(new org.eclipse.osgi.util.ManifestElement[size]);
      return result;
    } 
    throw new BundleException(NLS.bind("Invalid manifest header {0}: \"{1}\"", header, value), 3);
  }
  
  public static String[] getArrayFromList(String stringList) {
    String[] result = getArrayFromList(stringList, ",");
    return (result.length == 0) ? null : result;
  }
  
  public static String[] getArrayFromList(String stringList, String separator) {
    if (stringList == null || stringList.trim().length() == 0)
      return new String[0]; 
    ArrayList<String> list = new ArrayList();
    StringTokenizer tokens = new StringTokenizer(stringList, separator);
    while (tokens.hasMoreTokens()) {
      String token = tokens.nextToken().trim();
      if (token.length() != 0)
        list.add(token); 
    } 
    return list.<String>toArray(new String[list.size()]);
  }
  
  public static Map parseBundleManifest(InputStream manifest, Map<Object, Object> headers) throws IOException, BundleException {
    BufferedReader br;
    if (headers == null)
      headers = new HashMap<Object, Object>(); 
    try {
      br = new BufferedReader(new InputStreamReader(manifest, "UTF8"));
    } catch (UnsupportedEncodingException unsupportedEncodingException) {
      br = new BufferedReader(new InputStreamReader(manifest));
    } 
    try {
      String header = null;
      StringBuffer value = new StringBuffer(256);
      boolean firstLine = true;
      while (true) {
        String line = br.readLine();
        if (line == null || line.length() == 0) {
          if (!firstLine)
            headers.put(header, value.toString().trim()); 
          break;
        } 
        if (line.charAt(0) == ' ') {
          if (firstLine)
            throw new BundleException(NLS.bind("The manifest line \"{0}\" has an invalid leading space '' '' character.", line), 3); 
          value.append(line.substring(1));
          continue;
        } 
        if (!firstLine) {
          headers.put(header, value.toString().trim());
          value.setLength(0);
        } 
        int colon = line.indexOf(':');
        if (colon == -1)
          throw new BundleException(NLS.bind("The manifest line \"{0}\" is invalid; it has no colon '':'' character after the header key.", line), 3); 
        header = line.substring(0, colon).trim();
        value.append(line.substring(colon + 1));
        firstLine = false;
      } 
    } finally {
      try {
        br.close();
      } catch (IOException iOException) {}
    } 
    return headers;
  }
  
  public String toString() {
    Enumeration<String> attrKeys = getKeys();
    Enumeration<String> directiveKeys = getDirectiveKeys();
    if (attrKeys == null && directiveKeys == null)
      return this.value; 
    StringBuffer result = new StringBuffer(this.value);
    if (attrKeys != null)
      while (attrKeys.hasMoreElements()) {
        String key = attrKeys.nextElement();
        addValues(false, key, getAttributes(key), result);
      }  
    if (directiveKeys != null)
      while (directiveKeys.hasMoreElements()) {
        String key = directiveKeys.nextElement();
        addValues(true, key, getDirectives(key), result);
      }  
    return result.toString();
  }
  
  private void addValues(boolean directive, String key, String[] values, StringBuffer result) {
    if (values == null)
      return; 
    for (int i = 0; i < values.length; i++) {
      result.append(';').append(key);
      if (directive)
        result.append(':'); 
      result.append("=\"").append(values[i]).append('"');
    } 
  }
}
